import processing.serial.*;

Serial myPort;                       // active serial port
int xPos = 1;                        // horizontal position of the graph
float[] voutData = new float[4095];  // YPos
float[] timeData = new float[4095];  // XPos
int voutCount = 0;
boolean voutReady = false;
String recordLength = "24";          // time of flight record length in ms
String voutLevel = "3.3";            // VOUT voltage maximum level (3.3 or 5V)

void setup () {
  size(600, 300);                                       // set the window size
  println(Serial.list());                               // List all the available serial ports
  myPort = new Serial(this, Serial.list()[1], 115200);  // Open port used for the Launchpad at 115.2kBaud
  myPort.bufferUntil('\n');                             // do not generate a serialEvent() until newline character
  background(30);                                       // set inital background
}
void draw () {
  // waiting for serialEvent() trigger
  if (voutReady == true)
  {
    background(30);
    printVOUT();
    voutReady = false;
    voutCount = 0;
  }
}

void serialEvent (Serial myPort) {
  String inString = myPort.readStringUntil('\n'); // get the ASCII string
  if (inString != null) 
  {
    inString = trim(inString);                    // trim off any whitespace
    if(inString.contains("E") == true)            // end of data terminator received
    {      
      voutReady = true;
    }
    else
    {    
      float inByte = float(inString); // convert to an int and map to the screen height
      voutData[voutCount++] = inByte;
      if(voutCount >= voutData.length)
      {
        voutReady = true;
      }
    }
  }
}

void printVOUT()
{
  // remap points
  for(int i =0; i<voutCount; i++)
  {
    //convert string to float for 8-bit value
    voutData[i] = map(voutData[i], 0, 255, height*0.10, height*0.90); //YPOS
    timeData[i] = map(i, 0, voutCount, width*0.10, width*0.90);      //XPOS
  }
  
  // draw axes and labels
  rect(width * 0.10, height * 0.10, width * 0.80, height * 0.80);
  text("0", width * 0.08, height * 0.92);
  // y-axis
  line(width * 0.10, height * 0.10, width * 0.10, height * 0.90);
  text("VOUT (V)", width * 0.01, height * 0.5);
  text(voutLevel, width * 0.08, height * 0.08);
  //x-axis
  line(width * 0.10, height * 0.90, width * 0.90, height * 0.90);
  text("Time (ms)", width * 0.5, height * 0.99);
  text(recordLength, width * 0.92, height * 0.92);
  
  // draw results
  for(int i =0; i<voutCount-1; i++)
  {    
    stroke(0); // draw the line
    line(timeData[i], height - voutData[i], timeData[i+1], height-voutData[i+1]);
    // at the edge of the screen, go back to the beginning
    if (timeData[i] >= width) 
    {     
      return;
    }
  }
  
}
